from typing import Dict, List, Any
from configparser import ConfigParser
import json, os, xmltodict

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Jnfd(metaclass=Singleton):
    log: Logger
    cases: Dict[str, List[Dict[str, Any]]]
    servicePath: str

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')

        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        self.servicePath = conf.get('jnfd', 'path') if conf.has_option('jnfd', 'path') else '/NFDService'

        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

        servicestub.route(self.servicePath, method='POST', callback=self.req)

    def req(self):
        body = xmltodict.parse(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        method = list(body["soapenv:Envelope"]["soapenv:Body"].keys())[0]
        print(method)
        return self.createResponse(method)

    def createResponse(self, methodName: str):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            return None

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodName][0]:
            return self.cases[methodName][0].get('response')

        self.cases[methodName][0]['count'] -= 1
        response = self.cases[methodName][0].get('response')

        if self.cases[methodName][0]['count'] == 0:
            del self.cases[methodName][0]

        return response

instance = Jnfd()
