from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class RaiffeisenSbp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса RaiffeisenSbp')
        self.__updateCases()

        servicestub.route('/payment/v1/orders', method='POST', callback=self.orders)
        servicestub.route('/payment/v1/orders/<orderId>', method='GET', callback=self.ordersStatus)
        servicestub.route('/payments/v1/orders/<orderId>/refunds/<refundId>', method='POST', callback=self.refunds)
        servicestub.route('/payments/v1/orders/<orderId>/refunds/<refundId>', method='GET', callback=self.refundsStatus)

    def orders(self):
        self.log.info("Регистрация заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders')

    def ordersStatus(self, orderId=''):
        self.log.info("Получение статуса заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/statuses')

    def refunds(self, orderId='', refundId=''):
        self.log.info("Отмена заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/refunds')

    def refundsStatus(self, orderId='', refundId=''):
        self.log.info("Получение статуса отмены")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/refundsStatuses')

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))

        responseSuffix = self.cases[methodPath].get('responseSuffix', '')

        body = json.dumps(self.cases[methodPath].get('response' + responseSuffix))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["raiffeisensbp"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = RaiffeisenSbp()
